@@ -23,13 +23,6 @@ Gio._promisify(Gio.File.prototype, 'mount_enclosing_volume');
 
 const BACKGROUND_SCHEMA = 'org.gnome.desktop.background';
 
-const Hostname1Iface = '<node> \
-<interface name="org.freedesktop.hostname1"> \
-<property name="PrettyHostname" type="s" access="read" /> \
-</interface> \
-</node>';
-const Hostname1 = Gio.DBusProxy.makeProxyWrapper(Hostname1Iface);
-
 class PlaceInfo extends EventEmitter {
     constructor(...params) {
         super();
@@ -129,44 +122,20 @@ class PlaceInfo extends EventEmitter {
     }
 }
 
-class RootInfo extends PlaceInfo {
-    _init() {
-        super._init('devices', Gio.File.new_for_path('/'), _('Computer'));
+class NautilusSpecialInfo extends PlaceInfo {
+    constructor(file, name, icon) {
+        super('special', file, name, icon);
 
-        let busName = 'org.freedesktop.hostname1';
-        let objPath = '/org/freedesktop/hostname1';
-        new Hostname1(Gio.DBus.system, busName, objPath, (obj, error) => {
-            if (error)
-                return;
-
-            this._proxy = obj;
-            this._proxy.connectObject('g-properties-changed',
-                this._propertiesChanged.bind(this), this);
-            this._propertiesChanged(obj);
-        });
+        const appSystem = Shell.AppSystem.get_default();
+        this._app = appSystem.lookup_app('org.gnome.Nautilus.desktop');
     }
 
-    getIcon() {
-        return new Gio.ThemedIcon({name: 'drive-harddisk-symbolic'});
-    }
-
-    _propertiesChanged(proxy) {
-        // GDBusProxy will emit a g-properties-changed when hostname1 goes down
-        // ignore it
-        if (proxy.g_name_owner) {
-            this.name = proxy.PrettyHostname || _('Computer');
-            this.emit('changed');
-        }
-    }
-
-    destroy() {
-        this._proxy?.disconnectObject(this);
-        this._proxy = null;
-        super.destroy();
+    launch(timestamp) {
+        const launchContext = global.create_app_launch_context(timestamp, -1);
+        this._app.appInfo.launch([this.file], launchContext);
     }
 }
 
-
 class PlaceDeviceInfo extends PlaceInfo {
     _init(kind, mount) {
         this._mount = mount;
@@ -245,14 +214,6 @@ class PlaceVolumeInfo extends PlaceInfo {
     }
 }
 
-const DEFAULT_DIRECTORIES = [
-    GLib.UserDirectory.DIRECTORY_DOCUMENTS,
-    GLib.UserDirectory.DIRECTORY_PICTURES,
-    GLib.UserDirectory.DIRECTORY_MUSIC,
-    GLib.UserDirectory.DIRECTORY_DOWNLOAD,
-    GLib.UserDirectory.DIRECTORY_VIDEOS,
-];
-
 export class PlacesManager extends EventEmitter {
     constructor() {
         super();
@@ -267,6 +228,12 @@ export class PlacesManager extends EventEmitter {
         this._settings = new Gio.Settings({schema_id: BACKGROUND_SCHEMA});
         this._settings.connectObject('changed::show-desktop-icons',
             () => this._updateSpecials(), this);
+
+        this._privacySettings = new Gio.Settings({
+            schema_id: 'org.gnome.desktop.privacy',
+        });
+        this._privacySettings.connectObject('changed::remember-recent-files',
+            () => this._updateSpecials(), this);
         this._updateSpecials();
 
         /*
@@ -312,6 +279,9 @@ export class PlacesManager extends EventEmitter {
         this._settings?.disconnectObject(this);
         this._settings = null;
 
+        this._privacySettings.disconnectObject(this);
+        this._privacySettings = null;
+
         this._volumeMonitor.disconnectObject(this);
 
         if (this._monitor)
@@ -320,42 +290,68 @@ export class PlacesManager extends EventEmitter {
             GLib.source_remove(this._bookmarkTimeoutId);
     }
 
+    _shouldShowRecent() {
+        const vfs = Gio.Vfs.get_default();
+        const schemes = vfs.get_supported_uri_schemes();
+        return this._privacySettings.get_boolean('remember-recent-files') &&
+            schemes.includes('recent');
+    }
+
     _updateSpecials() {
         this._places.special.forEach(p => p.destroy());
         this._places.special = [];
 
-        let homePath = GLib.get_home_dir();
+        const appSystem = Shell.AppSystem.get_default();
+        const nautilusApp = appSystem.lookup_app('org.gnome.Nautilus.desktop');
+        const defaultFm = Gio.AppInfo.get_default_for_type('inode/directory', true);
+        const showNautilusSpecials =
+            nautilusApp && defaultFm && nautilusApp.appInfo.equal(defaultFm);
+
+        const homeFile = Gio.File.new_for_path(GLib.get_home_dir());
 
         this._places.special.push(new PlaceInfo(
             'special',
-            Gio.File.new_for_path(homePath),
+            homeFile,
             _('Home')));
 
-        let specials = [];
-        let dirs = DEFAULT_DIRECTORIES.slice();
-
-        if (this._settings.get_boolean('show-desktop-icons'))
-            dirs.push(GLib.UserDirectory.DIRECTORY_DESKTOP);
-
-        for (let i = 0; i < dirs.length; i++) {
-            let specialPath = GLib.get_user_special_dir(dirs[i]);
-            if (!specialPath || specialPath === homePath)
-                continue;
-
-            let file = Gio.File.new_for_path(specialPath), info;
-            try {
-                info = new PlaceInfo('special', file);
-            } catch (e) {
-                if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND))
-                    continue;
-                throw e;
-            }
-
-            specials.push(info);
+        if (this._shouldShowRecent()) {
+            this._places.special.push(new PlaceInfo(
+                'special',
+                Gio.File.new_for_uri('recent:///'),
+                _('Recent')));
         }
 
-        specials.sort((a, b) => GLib.utf8_collate(a.name, b.name));
-        this._places.special = this._places.special.concat(specials);
+        if (showNautilusSpecials) {
+            this._places.special.push(new NautilusSpecialInfo(
+                Gio.File.new_for_uri('starred:///'),
+                _('Starred'),
+                'starred-symbolic'));
+        }
+
+        if (this._settings.get_boolean('show-desktop-icons')) {
+            const desktopPath = GLib.get_user_special_dir(
+                GLib.UserDirectory.DIRECTORY_DESKTOP);
+            const desktopFile = desktopPath
+                ? Gio.File.new_for_path(desktopPath)
+                : null;
+
+            if (desktopFile && !desktopFile.equal(homeFile)) {
+                this._places.special.push(
+                    new PlaceInfo('special', desktopFile));
+            }
+        }
+
+        if (showNautilusSpecials) {
+            this._places.special.push(new NautilusSpecialInfo(
+                Gio.File.new_for_uri('x-network-view:///'),
+                _('Network'),
+                'network-workgroup-symbolic'));
+        }
+
+        this._places.special.push(new PlaceInfo(
+            'special',
+            Gio.File.new_for_uri('trash:///'),
+            _('Trash')));
 
         this.emit('special-updated');
     }
@@ -369,14 +365,6 @@ export class PlacesManager extends EventEmitter {
         this._places.network.forEach(p => p.destroy());
         this._places.network = [];
 
-        /* Add standard places */
-        this._places.devices.push(new RootInfo());
-        this._places.network.push(new PlaceInfo(
-            'network',
-            Gio.File.new_for_uri('network:///'),
-            _('Browse Network'),
-            'network-workgroup-symbolic'));
-
         /* first go through all connected drives */
         let drives = this._volumeMonitor.get_connected_drives();
         for (let i = 0; i < drives.length; i++) {
@@ -508,30 +496,12 @@ export class PlacesManager extends EventEmitter {
     }
 
     _addMount(kind, mount) {
-        let devItem;
-
-        try {
-            devItem = new PlaceDeviceInfo(kind, mount);
-        } catch (e) {
-            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND))
-                return;
-            throw e;
-        }
-
+        const devItem = new PlaceDeviceInfo(kind, mount);
         this._places[kind].push(devItem);
     }
 
     _addVolume(kind, volume) {
-        let volItem;
-
-        try {
-            volItem = new PlaceVolumeInfo(kind, volume);
-        } catch (e) {
-            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND))
-                return;
-            throw e;
-        }
-
+        const volItem = new PlaceVolumeInfo(kind, volume);
         this._places[kind].push(volItem);
     }
 
